@@ -33,15 +33,16 @@
 #include <errno.h>
 #include <m_string.h>
 #include "my_default.h"
+#include <m_ctype.h>
+#include "../mysys/mysys_priv.h"
 #include <hash.h>
 
 typedef void (*init_func_p)(const struct my_option *option, void *variable,
                             longlong value);
 
-static void default_reporter(enum loglevel level, const char *format, ...);
-my_error_reporter my_getopt_error_reporter= &default_reporter;
+my_error_reporter my_getopt_error_reporter= &my_message_local;
 
-static int findopt(char *, uint, const struct my_option **, const char **);
+static bool findopt(char *, uint, const struct my_option **);
 my_bool getopt_compare_strings(const char *, const char *, uint);
 static longlong getopt_ll(char *arg, const struct my_option *optp, int *err);
 static ulonglong getopt_ull(char *, const struct my_option *, int *);
@@ -143,7 +144,8 @@ static HASH *getopt_constraint_init(my_bool create)
     my_hash_init(&my_option_constraints, &my_charset_utf8_general_ci,
                  20, 0, 0,
                  (my_hash_get_key) getopt_constraint_get_name,
-                 (void (*)(void *))getopt_constraint_free, HASH_UNIQUE);
+                 (void (*)(void *))getopt_constraint_free, HASH_UNIQUE,
+                 PSI_NOT_INSTRUMENTED);
     my_option_constraints_inited= TRUE;
     return &my_option_constraints;
   }
@@ -170,7 +172,7 @@ static struct my_option_constraint *getopt_constraint_find(const char *name,
   if (length == 0)
     length= strlen(name);
 
-  normalized_name= my_strdup(name, MYF(MY_WME));
+  normalized_name= my_strdup(key_memory_defaults, name, MYF(MY_WME));
   for (pos= normalized_name; *pos; pos++)
   {
     if (*pos == '-')
@@ -183,7 +185,7 @@ static struct my_option_constraint *getopt_constraint_find(const char *name,
 
   if (!moc && create)
   {
-    moc= (struct my_option_constraint *) my_malloc(
+    moc= (struct my_option_constraint *) my_malloc(key_memory_defaults,
                                    sizeof(struct my_option_constraint),
                                    MYF(MY_WME | MY_ZEROFILL));
 
@@ -207,7 +209,8 @@ const void *getopt_constraint_get_min_value(const char *name, size_t length,
   moc= getopt_constraint_find(name, length, create > 0 ? TRUE : FALSE);
 
   if (moc && !moc->min_value && create)
-    moc->min_value= my_malloc(create, MYF(MY_WME | MY_ZEROFILL));
+    moc->min_value= my_malloc(key_memory_defaults, create,
+                              MYF(MY_WME | MY_ZEROFILL));
 
   if (moc && moc->min_value)
     return moc->min_value;
@@ -223,7 +226,8 @@ const void *getopt_constraint_get_max_value(const char *name, size_t length,
   moc= getopt_constraint_find(name, length, create > 0 ? TRUE : FALSE);
 
   if (moc && !moc->max_value && create)
-    moc->max_value= my_malloc(create, MYF(MY_WME | MY_ZEROFILL));
+    moc->max_value= my_malloc(key_memory_defaults, create,
+                              MYF(MY_WME | MY_ZEROFILL));
 
   if (moc && moc->max_value)
     return moc->max_value;
@@ -278,21 +282,6 @@ my_bool my_getopt_print_errors= 1;
 
 my_bool my_getopt_skip_unknown= 0;
 
-static void default_reporter(enum loglevel level,
-                             const char *format, ...)
-{
-  va_list args;
-  va_start(args, format);
-  if (level == WARNING_LEVEL)
-    fprintf(stderr, "%s", "Warning: ");
-  else if (level == INFORMATION_LEVEL)
-    fprintf(stderr, "%s", "Info: ");
-  vfprintf(stderr, format, args);
-  va_end(args);
-  fputc('\n', stderr);
-  fflush(stderr);
-}
-
 static my_getopt_value getopt_get_addr;
 
 void my_getopt_register_get_addr(my_getopt_value func_addr)
@@ -317,7 +306,7 @@ int handle_options(int *argc, char ***argv,
 		   const struct my_option *longopts,
                    my_get_one_option get_one_option)
 {
-  return my_handle_options(argc, argv, longopts, get_one_option, NULL);
+  return my_handle_options(argc, argv, longopts, get_one_option, NULL, FALSE);
 }
 
 union ull_dbl
@@ -412,22 +401,26 @@ double getopt_ulonglong2double(ulonglong v)
                              exit, argv [out] would contain all the remaining
                              unparsed options along with the matched command.
 
+  @param [in] ignore_unknown_option When set to TRUE, options are continued to
+                                    be read even when unknown options are
+				    encountered.
+
   @return error in case of ambiguous or unknown options,
           0 on success.
 */
 int my_handle_options(int *argc, char ***argv,
                       const struct my_option *longopts,
                       my_get_one_option get_one_option,
-                      const char **command_list)
+                      const char **command_list, my_bool ignore_unknown_option)
 {
-  uint UNINIT_VAR(opt_found), argvpos= 0, length;
-  my_bool end_of_options= 0, must_be_var;  
+  uint argvpos= 0, length;
+  my_bool end_of_options= 0, must_be_var;
   char **pos, **pos_end, *optend, *opt_str, key_name[FN_REFLEN];
-  const char *UNINIT_VAR(prev_found);
   const struct my_option *optp;
   void *value;
   int error, i, option_modifier_flags;
   my_bool is_cmdline_arg= 1;
+  bool opt_found;
 
   /* handle_options() assumes arg0 (program name) always exists */
   DBUG_ASSERT(argc && *argc >= 1);
@@ -453,7 +446,7 @@ int my_handle_options(int *argc, char ***argv,
   {
     char **first= pos;
     char *cur_arg= *pos;
-    opt_found= 0;
+    opt_found= false;
     if (!is_cmdline_arg && (my_getopt_is_args_separator(cur_arg)))
     {
       is_cmdline_arg= 1;
@@ -494,7 +487,7 @@ int my_handle_options(int *argc, char ***argv,
 	  or unknown option
 	*/
 	optp= longopts;
-	if (!(opt_found= findopt(opt_str, length, &optp, &prev_found)))
+	if (!(opt_found= findopt(opt_str, length, &optp)))
 	{
 	  /*
 	    Didn't find any matching option. Let's see if someone called
@@ -516,19 +509,9 @@ int my_handle_options(int *argc, char ***argv,
 		*/
 		opt_str+= special_opt_prefix_lengths[i] + 1;
                 length-= special_opt_prefix_lengths[i] + 1;
-		option_modifier_flags|= special_opt_flag[i];
-		if ((opt_found= findopt(opt_str, length, &optp, &prev_found)))
+                option_modifier_flags|= special_opt_flag[i];
+                if ((opt_found= findopt(opt_str, length, &optp)))
 		{
-		  if (opt_found > 1)
-		  {
-		    if (my_getopt_print_errors)
-                      my_getopt_error_reporter(ERROR_LEVEL,
-                                               "%s: ambiguous option '--%s-%s' (--%s-%s)",
-                                               my_progname, special_opt_prefix[i],
-                                               opt_str, special_opt_prefix[i],
-                                               prev_found);
-		    return EXIT_AMBIGUOUS_OPTION;
-		  }
 		  switch (special_opt_flag[i]) {
 		  case OPT_SKIP:
 		  case OPT_DISABLE: /* fall through */
@@ -578,58 +561,35 @@ int my_handle_options(int *argc, char ***argv,
 	    if (must_be_var)
 	    {
 	      if (my_getopt_print_errors)
-                my_getopt_error_reporter((option_modifier_flags & OPT_LOOSE) ? 
-                                           WARNING_LEVEL : ERROR_LEVEL,
-                                         "%s: unknown variable '%s'",
-                                         my_progname, cur_arg);
-	      if (!(option_modifier_flags  & OPT_LOOSE))
+                my_getopt_error_reporter((option_modifier_flags & OPT_LOOSE) ?
+                                         WARNING_LEVEL : ERROR_LEVEL,
+                                         "unknown variable '%s'", cur_arg);
+              if (!(option_modifier_flags  & OPT_LOOSE))
 		return EXIT_UNKNOWN_VARIABLE;
 	    }
 	    else
 	    {
 	      if (my_getopt_print_errors)
-                my_getopt_error_reporter((option_modifier_flags  & OPT_LOOSE) ? 
-                                           WARNING_LEVEL : ERROR_LEVEL,
-                                         "%s: unknown option '--%s'", 
-                                         my_progname, cur_arg);
-	      if (!(option_modifier_flags  & OPT_LOOSE))
+                my_getopt_error_reporter((option_modifier_flags  & OPT_LOOSE) ?
+                                         WARNING_LEVEL : ERROR_LEVEL,
+                                         "unknown option '--%s'", cur_arg);
+              if (!(option_modifier_flags  & OPT_LOOSE))
 		return EXIT_UNKNOWN_OPTION;
 	    }
-	    if (option_modifier_flags  & OPT_LOOSE)
+            if (option_modifier_flags  & OPT_LOOSE)
 	    {
 	      (*argc)--;
 	      continue;
 	    }
 	  }
 	}
-	if (opt_found > 1)
-	{
-	  if (must_be_var)
-	  {
-	    if (my_getopt_print_errors)
-              my_getopt_error_reporter(ERROR_LEVEL,
-                                       "%s: variable prefix '%s' is not unique",
-                                       my_progname, opt_str);
-	    return EXIT_VAR_PREFIX_NOT_UNIQUE;
-	  }
-	  else
-	  {
-	    if (my_getopt_print_errors)
-              my_getopt_error_reporter(ERROR_LEVEL,
-                                       "%s: ambiguous option '--%s' (%s, %s)",
-                                       my_progname, opt_str, prev_found, 
-                                       optp->name);
-	    return EXIT_AMBIGUOUS_OPTION;
-	  }
-	}
 	if ((optp->var_type & GET_TYPE_MASK) == GET_DISABLED)
 	{
 	  if (my_getopt_print_errors)
-	    fprintf(stderr,
-		    "%s: %s: Option '%s' used, but is disabled\n", my_progname,
-		    (option_modifier_flags  & OPT_LOOSE) ?
-                    "WARNING" : "ERROR", opt_str);
-	  if (option_modifier_flags  & OPT_LOOSE)
+            my_message_local((option_modifier_flags  & OPT_LOOSE) ? WARNING_LEVEL : ERROR_LEVEL,
+                             "%s: Option '%s' used, but is disabled",
+                             my_progname, opt_str);
+          if (option_modifier_flags  & OPT_LOOSE)
 	  {
 	    (*argc)--;
 	    continue;
@@ -638,7 +598,7 @@ int my_handle_options(int *argc, char ***argv,
 	}
         error= 0;
 	value= optp->var_type & GET_ASK_ADDR ?
-	  (*getopt_get_addr)(key_name, (uint) strlen(key_name), optp, &error) :
+	  (*getopt_get_addr)(key_name, strlen(key_name), optp, &error) :
           optp->value;
         if (error)
           return error;
@@ -719,19 +679,19 @@ int my_handle_options(int *argc, char ***argv,
       {
 	for (optend= cur_arg; *optend; optend++)
 	{
-	  opt_found= 0;
+	  opt_found= false;
 	  for (optp= longopts; optp->name; optp++)
 	  {
 	    if (optp->id && optp->id == (int) (uchar) *optend)
 	    {
 	      /* Option recognized. Find next what to do with it */
-	      opt_found= 1;
+	      opt_found= true;
 	      if ((optp->var_type & GET_TYPE_MASK) == GET_DISABLED)
 	      {
 		if (my_getopt_print_errors)
-		  fprintf(stderr,
-			  "%s: ERROR: Option '-%c' used, but is disabled\n",
-			  my_progname, optp->id);
+                  my_message_local(ERROR_LEVEL,
+                                   "%s: Option '-%c' used, but is disabled",
+                                   my_progname, optp->id);
 		return EXIT_OPTION_DISABLED;
 	      }
 	      if ((optp->var_type & GET_TYPE_MASK) == GET_BOOL &&
@@ -877,9 +837,8 @@ void print_cmdline_password_warning()
 
   if (!password_warning_announced)
   {
-    fprintf(stderr, "Warning: Using a password on the command line "
-            "interface can be insecure.\n");
-    (void) fflush(stderr);
+    my_message_local(WARNING_LEVEL, "Using a password on the command line "
+                                    "interface can be insecure.");
     password_warning_announced= TRUE;
   }
 }
@@ -887,7 +846,7 @@ void print_cmdline_password_warning()
 
 /*
   Clean up any allocations made during handle_options on shutdown.
-*/
+ */
 void my_handle_options_end()
 {
   if (my_option_constraints_inited)
@@ -898,26 +857,31 @@ void my_handle_options_end()
 }
 
 
-/*
-  function: check_struct_option
+/**
+   @brief Check for struct options
 
-  Arguments: Current argument under processing from argv and a variable
-  where to store the possible key name.
-
-  Return value: In case option is a struct option, returns a pointer to
-  the current argument at the position where the struct option (key_name)
-  ends, the next character after the dot. In case argument is not a struct
-  option, returns a pointer to the argument.
+  @param[in]   cur_arg     Current argument under processing from argv
+  @param[in]   key_name    variable where to store the possible key name 
 
+  @details
+  In case option is a struct option, returns a pointer to the current
+  argument at the position where the struct option (key_name) ends, the
+  next character after the dot. In case argument is not a struct option,
+  returns a pointer to the argument.
   key_name will hold the name of the key, or 0 if not found.
+
+  @return char*
+  If struct option     Pointer to next character after dot.
+  If no struct option  Pointer to the argument
 */
 
 static char *check_struct_option(char *cur_arg, char *key_name)
 {
-  char *ptr, *end;
-
-  ptr= strcend(cur_arg + 1, '.'); /* Skip the first character */
-  end= strcend(cur_arg, '=');
+  char *dot_pos, *equal_pos, *space_pos;
+ 
+  dot_pos= strcend(cur_arg + 1, '.'); /* Skip the first character */
+  equal_pos= strcend(cur_arg, '=');
+  space_pos= strcend(cur_arg, ' ');
 
   /* 
      If the first dot is after an equal sign, then it is part
@@ -926,12 +890,12 @@ static char *check_struct_option(char *cur_arg, char *key_name)
      NULL, or the character right before equal sign is the first
      dot found, the option is not a struct option.
   */
-  if (end - ptr > 1)
+  if ((equal_pos > dot_pos) && (space_pos > dot_pos))
   {
-    uint len= (uint) (ptr - cur_arg);
+    size_t len= (uint) (dot_pos - cur_arg);
     set_if_smaller(len, FN_REFLEN-1);
     strmake(key_name, cur_arg, len);
-    return ++ptr;
+    return ++dot_pos;
   }
   else
   {
@@ -1005,70 +969,104 @@ static size_t setval_get_size_for_value_type(int value_type)
 static int setval(const struct my_option *opts, const void *value,
                   char *argument, int option_modifier_flags)
 {
-  int err= 0, res= 0, value_type;
+  int err= 0, res= 0;
   bool error= 0;
+  ulong var_type= opts->var_type & GET_TYPE_MASK;
 
   if (!argument)
     argument= enabled_my_option;
 
   if (option_modifier_flags & OPT_MINIMUM)
   {
-      value_type= (opts->var_type & GET_TYPE_MASK);
-      value= getopt_constraint_get_min_value(opts->name, 0,
-                                 setval_get_size_for_value_type(value_type));
-      if (!value)
-      {
-        my_getopt_error_reporter(ERROR_LEVEL,
+     value= getopt_constraint_get_min_value(opts->name, 0,
+                                 setval_get_size_for_value_type(var_type));
+     if (!value)
+     {
+       my_getopt_error_reporter(ERROR_LEVEL,
                                "%s: Minimum value of '%s' cannot be set",
                                my_progname, opts->name);
-        return EXIT_NO_PTR_TO_VARIABLE;
-      }
+       return EXIT_NO_PTR_TO_VARIABLE;
+     }
   }
   else if (option_modifier_flags & OPT_MAXIMUM)
   {
-      value_type= (opts->var_type & GET_TYPE_MASK);
-      value= getopt_constraint_get_max_value(opts->name, 0,
-                                 setval_get_size_for_value_type(value_type));
-      if (!value)
-      {
-        my_getopt_error_reporter(ERROR_LEVEL,
+     value= getopt_constraint_get_max_value(opts->name, 0,
+                                 setval_get_size_for_value_type(var_type));
+     if (!value)
+     {
+       my_getopt_error_reporter(ERROR_LEVEL,
                                "%s: Maximum value of '%s' cannot be set",
                                my_progname, opts->name);
-        return EXIT_NO_PTR_TO_VARIABLE;
-      }
+       return EXIT_NO_PTR_TO_VARIABLE;
+     }
   }
   else if (option_modifier_flags & OPT_READONLY)
   {
-      value_type= GET_BOOL;
-      value= getopt_constraint_get_readonly_value(opts->name, 0, TRUE);
-      if (!value)
-      {
-        my_getopt_error_reporter(ERROR_LEVEL,
+    var_type= GET_BOOL;
+    value= getopt_constraint_get_readonly_value(opts->name, 0, TRUE);
+    if (!value)
+    {
+      my_getopt_error_reporter(ERROR_LEVEL,
                                "%s: Readonly value of '%s' cannot be set",
                                my_progname, opts->name);
-        return EXIT_NO_PTR_TO_VARIABLE;
-      }
-  } 
+      return EXIT_NO_PTR_TO_VARIABLE;
+    }
+  }
   else if (option_modifier_flags & OPT_HIDDEN)
   {
-      value_type= GET_BOOL;
-      value= getopt_constraint_get_hidden_value(opts->name, 0, TRUE);
-      if (!value)
-      {
-        my_getopt_error_reporter(ERROR_LEVEL,
+    var_type= GET_BOOL;
+    value= getopt_constraint_get_hidden_value(opts->name, 0, TRUE);
+    if (!value)
+    {
+      my_getopt_error_reporter(ERROR_LEVEL,
                                "%s: Hidden value of '%s' cannot be set",
                                my_progname, opts->name);
-        return EXIT_NO_PTR_TO_VARIABLE;
-      }
+      return EXIT_NO_PTR_TO_VARIABLE;
+    }
   }
-  else
-  { 
-    value_type= (opts->var_type & GET_TYPE_MASK);
+
+  /*
+    Thus check applies only to options that have a defined value
+    storage pointer.
+    We do it for numeric types only, as empty value is a valid
+    option for strings (the only way to reset back to default value).
+    Note: it does not relate to OPT_ARG/REQUIRED_ARG/NO_ARG, since
+    --param="" is not generally the same as --param.
+    TODO: Add an option definition flag to signify whether empty value
+    (i.e. --param="") is an acceptable value or an error and extend
+    the check to all options.
+  */
+  if (!*argument &&
+      (
+       var_type == GET_INT ||
+       var_type == GET_UINT ||
+       var_type == GET_LONG ||
+       var_type == GET_ULONG ||
+       var_type == GET_LL ||
+       var_type == GET_ULL ||
+       var_type == GET_DOUBLE ||
+       var_type == GET_ENUM
+       )
+      )
+  {
+    if (strncmp(opts->name, "port", 10) == 0)
+    {
+        my_getopt_error_reporter(WARNING_LEVEL,
+                             "%s: Empty value for '%s' specified. Will throw an error in future versions",
+                             my_progname, opts->name);
+    }
+    else
+    {
+        my_getopt_error_reporter(ERROR_LEVEL,
+                             "%s: Empty value for '%s' specified",
+                             my_progname, opts->name);
+        return EXIT_ARGUMENT_REQUIRED;
+    }
   }
-  
+
   if (value)
   {
-    switch (value_type)
+    switch (var_type)
     {
     case GET_BOOL: /* If argument differs from 0, enable option, else disable */
       *((my_bool*) value)= get_bool_argument(opts, argument, &error);
@@ -1108,7 +1106,8 @@ static int setval(const struct my_option *opts, const void *value,
       if (argument == enabled_my_option)
         break; /* string options don't use this default of "1" */
       my_free(*((char**) value));
-      if (!(*((char**) value)= my_strdup(argument, MYF(MY_WME))))
+      if (!(*((char**) value)= my_strdup(key_memory_defaults,
+                                         argument, MYF(MY_WME))))
       {
         res= EXIT_OUT_OF_MEMORY;
         goto ret;
@@ -1164,7 +1163,7 @@ static int setval(const struct my_option *opts, const void *value,
         *((ulonglong*)value)=
               find_set_from_flags(opts->typelib, opts->typelib->count, 
                                   *(ulonglong *)value, opts->def_value,
-                                  argument, strlen(argument),
+                                  argument, (uint)strlen(argument),
                                   &error, &error_len);
         if (error)
         {
@@ -1193,69 +1192,32 @@ ret:
 }
 
 
-/* 
+/**
   Find option
 
-  SYNOPSIS
-    findopt()
-    optpat	Prefix of option to find (with - or _)
-    length	Length of optpat
-    opt_res	Options
-    ffname	Place for pointer to first found name
-
   IMPLEMENTATION
-    Go through all options in the my_option struct. Return number
-    of options found that match the pattern and in the argument
-    list the option found, if any. In case of ambiguous option, store
-    the name in ffname argument
+    Go through all options in the my_option struct. Return true
+    if an option is found. sets opt_res to the option found, if any. 
 
-    RETURN
-    0    No matching options
-    #   Number of matching options
-        ffname points to first matching option
+    @param         optpat   name of option to find (with - or _)
+    @param         length   Length of optpat
+    @param[in,out] opt_res  Options
+
+    @retval false    No matching options
+    @retval true     Found an option
 */
 
-static int findopt(char *optpat, uint length,
-		   const struct my_option **opt_res,
-		   const char **ffname)
+static bool findopt(char *optpat, uint length,
+		   const struct my_option **opt_res)
 {
-  uint count;
-  const struct my_option *opt= *opt_res;
-  my_bool is_prefix= FALSE;
-
-  for (count= 0; opt->name; opt++)
-  {
-    if (!getopt_compare_strings(opt->name, optpat, length)) /* match found */
+  for (const struct my_option *opt= *opt_res; opt->name; opt++)
+    if (!getopt_compare_strings(opt->name, optpat, length) &&
+        !opt->name[length])
     {
       (*opt_res)= opt;
-      if (!opt->name[length])		/* Exact match */
-	return 1;
-
-      if (!count)
-      {
-        /* We only need to know one prev */
-	count= 1;
-	*ffname= opt->name;
-        if (opt->name[length])
-          is_prefix= TRUE;
-      }
-      else if (strcmp(*ffname, opt->name))
-      {
-	/*
-	  The above test is to not count same option twice
-	  (see mysql.cc, option "help")
-	*/
-	count++;
-      }
+      return true;
     }
-  }
-  if (is_prefix && count == 1)
-    my_getopt_error_reporter(WARNING_LEVEL,
-                             "Using unique option prefix %.*s instead of %s "
-                             "is deprecated and will be removed in a future "
-                             "release. Please use the full name instead.",
-                             length, optpat, *ffname);
-  return count;
+  return false;
 }
 
 
@@ -1292,7 +1254,7 @@ static longlong eval_num_suffix(char *argument, int *error, char *option_name)
   
   *error= 0;
   errno= 0;
-  num= strtoll(argument, &endchar, 10);
+  num= my_strtoll(argument, &endchar, 10);
   if (errno == ERANGE)
   {
     my_getopt_error_reporter(ERROR_LEVEL,
@@ -1308,35 +1270,37 @@ static longlong eval_num_suffix(char *argument, int *error, char *option_name)
     num*= 1024L * 1024L * 1024L;
   else if (*endchar)
   {
-    fprintf(stderr,
-	    "Unknown suffix '%c' used for variable '%s' (value '%s')\n",
-	    *endchar, option_name, argument);
+    my_message_local(ERROR_LEVEL,
+                     "Unknown suffix '%c' used for variable '%s' (value '%s')",
+                     *endchar, option_name, argument);
     *error= 1;
     return 0;
   }
   return num;
 }
 
-/* 
+/**
   function: eval_num_suffix_ull
-
-  Transforms a number with a suffix to unsigned longlong number. Suffix can
+  This is the same as eval_num_suffix, but is meant for unsigned long long
+  values. Transforms an unsigned number with a suffix to real number. Suffix can
   be k|K for kilo, m|M for mega or g|G for giga.
+  @param [IN]        argument      argument value for option_name
+  @param [IN, OUT]   error         error no.
+  @param [IN]        option_name   name of option
 */
 
-static ulonglong eval_num_suffix_ull(char *argument, int *error,
-                                     char *option_name)
+static ulonglong eval_num_suffix_ull(char *argument, int *error, char *option_name)
 {
   char *endchar;
   ulonglong num;
 
   *error= 0;
   errno= 0;
-  num= strtoull(argument, &endchar, 10);
+  num= my_strtoull(argument, &endchar, 10);
   if (errno == ERANGE)
   {
     my_getopt_error_reporter(ERROR_LEVEL,
-                             "Incorrect unsigned integer value:'%s'", argument);
+                             "Incorrect unsigned integer value: '%s'", argument);
     *error= 1;
     return 0;
   }
@@ -1348,9 +1312,9 @@ static ulonglong eval_num_suffix_ull(char *argument, int *error,
     num*= 1024L * 1024L * 1024L;
   else if (*endchar)
   {
-    fprintf(stderr,
-	    "Unknown suffix '%c' used for variable '%s' (value '%s')\n",
-	    *endchar, option_name, argument);
+    my_message_local(ERROR_LEVEL,
+                     "Unknown suffix '%c' used for variable '%s' (value '%s')",
+                     *endchar, option_name, argument);
     *error= 1;
     return 0;
   }
@@ -1389,13 +1353,13 @@ ulonglong max_of_int_range(int var_type)
   case GET_LONG:
     return LONG_MAX;
   case GET_LL:
-    return LONGLONG_MAX;
+    return LLONG_MAX;
   case GET_UINT:
     return UINT_MAX;
   case GET_ULONG:
     return ULONG_MAX;
   case GET_ULL:
-    return ULONGLONG_MAX;
+    return ULLONG_MAX;
   default:
     DBUG_ASSERT(0);
     return 0;
@@ -1454,9 +1418,9 @@ longlong getopt_ll_limit_value(longlong num, const struct my_option *optp,
 
 static inline my_bool is_negative_num(char* num)
 {
-  while (my_isspace(&my_charset_latin1, *num))
+  while (my_isspace(&my_charset_latin1, *num)) 
     num++;
-
+  
   return (*num == '-');
 }
 
@@ -1482,7 +1446,7 @@ static ulonglong getopt_ull(char *arg, const struct my_option *optp, int *err)
   }
   else
     num= eval_num_suffix_ull(arg, err, (char*) optp->name);
-
+  
   return getopt_ull_limit_value(num, optp, NULL);
 }
 
@@ -1496,7 +1460,7 @@ ulonglong getopt_ull_limit_value(ulonglong num, const struct my_option *optp,
   const ulonglong max_of_type=
     max_of_int_range(optp->var_type & GET_TYPE_MASK);
 
-  if ((ulonglong) num > (ulonglong) optp->max_value &&
+  if (num > (ulonglong) optp->max_value &&
       optp->max_value) /* if max value is not set -> no upper limit */
   {
     num= (ulonglong) optp->max_value;
@@ -1621,10 +1585,10 @@ static void init_one_value(const struct my_option *option, void *variable,
     *((ulong*) variable)= (ulong) getopt_ull_limit_value((ulong) value, option, NULL);
     break;
   case GET_LL:
-    *((longlong*) variable)= (longlong) getopt_ll_limit_value((longlong) value, option, NULL);
+    *((longlong*) variable)= getopt_ll_limit_value(value, option, NULL);
     break;
   case GET_ULL:
-    *((ulonglong*) variable)= (ulonglong) getopt_ull_limit_value((ulonglong) value, option, NULL);
+    *((ulonglong*) variable)= getopt_ull_limit_value((ulonglong) value, option, NULL);
     break;
   case GET_SET:
   case GET_FLAGSET:
@@ -1655,7 +1619,8 @@ static void init_one_value(const struct my_option *option, void *variable,
     {
       char **pstr= (char **) variable;
       my_free(*pstr);
-      *pstr= my_strdup((char*) (intptr) value, MYF(MY_WME));
+      *pstr= my_strdup(key_memory_defaults,
+                       (char*) (intptr) value, MYF(MY_WME));
     }
     break;
   default: /* dummy default to avoid compiler warnings */
@@ -1732,12 +1697,18 @@ static void init_variables(const struct my_option *options,
   DBUG_VOID_RETURN;
 }
 
-/** Prints variable or option name, replacing _ with - */
-static uint print_name(const struct my_option *optp)
+/**
+  Prints variable or option name, replacing _ with - to given file stream
+  parameter (by default to stdout).
+  @param [IN] optp      my_option parameter
+  @param [IN] file      stream where the output of optp parameter name
+                        goes (by default to stdout).
+*/
+static uint print_name(const struct my_option *optp, FILE* file = stdout)
 {
   const char *s= optp->name;
   for (;*s;s++)
-    putchar(*s == '_' ? '-' : *s);
+    putc(*s == '_' ? '-' : *s, file);
   return s - optp->name;
 }
 
@@ -1830,16 +1801,27 @@ void my_print_help(const struct my_option *options)
   }
 }
 
-
-/*
-  function: my_print_options
-
-  Print variables.
+/**
+ function: my_print_variables
+ Print variables.
+ @param [IN] options    my_option list
 */
-
 void my_print_variables(const struct my_option *options)
 {
-  uint name_space= 34, length, nr;
+  my_print_variables_ex(options, stdout);
+}
+
+/**
+  function: my_print_variables_ex
+  Print variables to given file parameter stream (by default to stdout).
+  @param [IN] options    my_options list
+  @param [IN] file       stream where the output goes.
+*/
+
+void my_print_variables_ex(const struct my_option *options, FILE* file)
+{
+  uint name_space= 34, nr;
+  size_t length;
   ulonglong llvalue;
   char buff[255];
   const struct my_option *optp;
@@ -1851,82 +1833,83 @@ void my_print_variables(const struct my_option *options)
       name_space= length;
   }
 
-  printf("\nVariables (--variable-name=value)\n");
-  printf("%-*s%s", name_space, "and boolean options {FALSE|TRUE}",
-         "Value (after reading options)\n");
+  fprintf(file, "\nVariables (--variable-name=value)\n");
+  fprintf(file, "%-*s%s", name_space, "and boolean options {FALSE|TRUE}",
+          "Value (after reading options)\n");
   for (length=1; length < 75; length++)
-    putchar(length == name_space ? ' ' : '-');
-  putchar('\n');
-  
+    putc(length == name_space ? ' ' : '-', file);
+  putc('\n', file);
+
   for (optp= options; optp->name; optp++)
   {
     void *value= (optp->var_type & GET_ASK_ADDR ?
 		  (*getopt_get_addr)("", 0, optp, 0) : optp->value);
     if (value)
     {
-      length= print_name(optp);
+      length= print_name(optp, file);
       for (; length < name_space; length++)
-	putchar(' ');
+        putc(' ', file);
       switch ((optp->var_type & GET_TYPE_MASK)) {
       case GET_SET:
         if (!(llvalue= *(ulonglong*) value))
-	  printf("%s\n", "");
+          fprintf(file, "%s\n", "");
 	else
         for (nr= 0; llvalue && nr < optp->typelib->count; nr++, llvalue >>=1)
 	{
 	  if (llvalue & 1)
-            printf( llvalue > 1 ? "%s," : "%s\n", get_type(optp->typelib, nr));
+            fprintf(file, llvalue > 1 ? "%s," : "%s\n",
+                    get_type(optp->typelib, nr));
 	}
 	break;
       case GET_FLAGSET:
         llvalue= *(ulonglong*) value;
         for (nr= 0; llvalue && nr < optp->typelib->count; nr++, llvalue >>=1)
 	{
-          printf("%s%s=", (nr ? "," : ""), get_type(optp->typelib, nr));
-	  printf(llvalue & 1 ? "on" : "off");
+          fprintf(file, "%s%s=", (nr ? "," : ""), get_type(optp->typelib, nr));
+          fprintf(file, llvalue & 1 ? "on" : "off");
 	}
-        printf("\n");
-	break;
+        fprintf(file, "\n");
+        break;
       case GET_ENUM:
-        printf("%s\n", get_type(optp->typelib, *(ulong*) value));
-	break;
+        fprintf(file, "%s\n", get_type(optp->typelib, *(ulong*) value));
+        break;
       case GET_STR:
       case GET_PASSWORD:
       case GET_STR_ALLOC:                    /* fall through */
-	printf("%s\n", *((char**) value) ? *((char**) value) :
-	       "(No default value)");
-	break;
+        fprintf(file, "%s\n", *((char**) value) ? *((char**) value) :
+                "(No default value)");
+        break;
       case GET_BOOL:
-	printf("%s\n", *((my_bool*) value) ? "TRUE" : "FALSE");
-	break;
+        fprintf(file, "%s\n", *((my_bool*) value) ? "TRUE" : "FALSE");
+        break;
       case GET_INT:
-	printf("%d\n", *((int*) value));
-	break;
+        fprintf(file, "%d\n", *((int*) value));
+        break;
       case GET_UINT:
-	printf("%d\n", *((uint*) value));
-	break;
+        fprintf(file, "%d\n", *((uint*) value));
+        break;
       case GET_LONG:
-	printf("%ld\n", *((long*) value));
-	break;
+        fprintf(file, "%ld\n", *((long*) value));
+        break;
       case GET_ULONG:
-	printf("%lu\n", *((ulong*) value));
-	break;
+        fprintf(file, "%lu\n", *((ulong*) value));
+        break;
       case GET_LL:
-	printf("%s\n", llstr(*((longlong*) value), buff));
-	break;
+        fprintf(file, "%s\n", llstr(*((longlong*) value), buff));
+        break;
       case GET_ULL:
 	longlong2str(*((ulonglong*) value), buff, 10);
-	printf("%s\n", buff);
-	break;
+        fprintf(file, "%s\n", buff);
+        break;
       case GET_DOUBLE:
-	printf("%g\n", *(double*) value);
-	break;
+        fprintf(file, "%g\n", *(double*) value);
+        break;
       case GET_NO_ARG:
-	printf("(No default value)\n");
-	break;
+        fprintf(file, "(No default value)\n");
+        break;
       default:
-	printf("(Disabled)\n");
-	break;
+        fprintf(file, "(Disabled)\n");
+        break;
       }
     }
   }
