@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2010-2015 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2020 Codership Oy <info@codership.com>
 //
 
 #include "replicator_smm.hpp"
@@ -45,7 +45,7 @@ ReplicatorSMM::state_transfer_required(const wsrep_view_info_t& view_info)
                     abort();
                 }
 
-                return (local_seqno != group_seqno);
+                return (local_seqno < group_seqno);
             }
         }
 
@@ -814,10 +814,11 @@ ReplicatorSMM::request_state_transfer (void* recv_ctx,
         st_.mark_unsafe();
     }
 
+    GU_DBUG_SYNC_WAIT("before_send_state_request");
+
     // We must set SST state to "wait" before
     // sending request, to avoid racing condition
     // in the sst_received.
-
     sst_state_ = SST_WAIT;
 
     // We should not wait for completion of the SST or to handle it
@@ -840,6 +841,9 @@ ReplicatorSMM::request_state_transfer (void* recv_ctx,
     GU_DBUG_SYNC_WAIT("after_send_state_request");
 
     state_.shift_to(S_JOINING);
+
+    GU_DBUG_SYNC_WAIT("after_shift_to_joining");
+
     /* while waiting for state transfer to complete is a good point
      * to reset gcache, since it may involve some IO too */
     gcache_.seqno_reset(to_gu_uuid(group_uuid), group_seqno);
@@ -903,7 +907,8 @@ ReplicatorSMM::request_state_transfer (void* recv_ctx,
                 commit_monitor_.set_initial_position(sst_seqno_);
             }
 
-            log_debug << "Installed new state: " << state_uuid_ << ":" << sst_seqno_;
+            log_debug << "Installed new state: " << state_uuid_ << ":"
+                      << sst_seqno_;
         }
     }
     else
