@@ -1,13 +1,20 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
@@ -837,6 +844,13 @@ bool lock_schema_name(THD *thd, const char *db)
                                      thd->variables.lock_wait_timeout))
     return TRUE;
 
+  /*
+    Now when we have protection against concurrent change of read_only
+    option we can safely re-check its value.
+  */
+  if (check_readonly(thd, true))
+    return TRUE;
+
   DEBUG_SYNC(thd, "after_wait_locked_schema_name");
   return FALSE;
 }
@@ -889,6 +903,13 @@ bool lock_tablespace_name(THD *thd, const char *tablespace)
                                      thd->variables.lock_wait_timeout))
     return true;
 
+  /*
+    Now when we have protection against concurrent change of read_only
+    option we can safely re-check its value.
+  */
+  if (check_readonly(thd, true))
+    return TRUE;
+
   DEBUG_SYNC(thd, "after_wait_locked_tablespace_name");
   return false;
 }
@@ -1006,6 +1027,13 @@ bool lock_object_name(THD *thd, MDL_key::enum_mdl_namespace mdl_type,
                                      thd->variables.lock_wait_timeout))
     return TRUE;
 
+  /*
+    Now when we have protection against concurrent change of read_only
+    option we can safely re-check its value.
+  */
+  if (check_readonly(thd, true))
+    return TRUE;
+
   DEBUG_SYNC(thd, "after_wait_locked_pname");
   return FALSE;
 }
@@ -1276,7 +1304,11 @@ bool Global_read_lock::make_global_read_lock_block_commit(THD *thd)
     int rcode;
     WSREP_DEBUG("Running implicit desync for node from FTWRL");
     rcode = wsrep->desync(wsrep);
-    if (rcode != WSREP_OK)
+    if (rcode == WSREP_TRX_FAIL && strcmp(wsrep_cluster_status, "Primary") != 0)
+    {
+      WSREP_DEBUG("desync failed while non-Primary, ignoring failure");
+    }
+    else if (rcode != WSREP_OK)
     {
       WSREP_WARN("FTWRL desync failed %d for schema: %s, query: %s",
                  rcode, (thd->db().length ? thd->db().str : "(null)"), WSREP_QUERY(thd));
@@ -1403,8 +1435,8 @@ bool Global_backup_lock::acquire(THD *thd)
 
   DBUG_ENTER("Global_backup_lock::acquire");
 
-  DBUG_ASSERT(m_lock == NULL &&
-              !thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "",
+  DBUG_ASSERT(m_lock == NULL);
+  DBUG_ASSERT(!thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "",
                                                             "", MDL_SHARED));
 
   MDL_REQUEST_INIT(&mdl_request, m_namespace, "", "", MDL_SHARED,
@@ -1429,8 +1461,8 @@ void Global_backup_lock::release(THD *thd)
 {
   DBUG_ENTER("Global_backup_lock::release");
 
-  DBUG_ASSERT(m_lock != NULL &&
-              thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
+  DBUG_ASSERT(m_lock != NULL);
+  DBUG_ASSERT(thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
                                                            MDL_SHARED));
 
   thd->mdl_context.release_lock(m_lock);
@@ -1531,8 +1563,8 @@ void Global_backup_lock::release_protection(THD *thd)
 {
   DBUG_ENTER("Global_backup_lock::release_protection");
 
-  DBUG_ASSERT(m_prot_lock != NULL &&
-              thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
+  DBUG_ASSERT(m_prot_lock != NULL);
+  DBUG_ASSERT(thd->mdl_context.owns_equal_or_stronger_lock(m_namespace, "", "",
                                              MDL_INTENTION_EXCLUSIVE));
 
   thd->mdl_context.release_lock(m_prot_lock);
