@@ -1,15 +1,22 @@
 #ifndef MDL_H
 #define MDL_H
-/* Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
@@ -120,10 +127,15 @@ public:
     Notify/get permission from interested storage engines before acquiring
     exclusive lock for the key.
 
+    The returned argument 'victimized' specify reason for lock
+    not granted. If 'true', lock was refused in an attempt to 
+    resolve a possible MDL->GSL deadlock. Locking may then be retried.
+
     @return False if notification was successful and it is OK to acquire lock,
             True if one of SEs asks to abort lock acquisition.
   */
-  virtual bool notify_hton_pre_acquire_exclusive(const MDL_key *mdl_key) = 0;
+  virtual bool notify_hton_pre_acquire_exclusive(const MDL_key *mdl_key,
+                                                 bool *victimized) = 0;
   /**
     Notify interested storage engines that we have just released exclusive
     lock for the key.
@@ -493,6 +505,7 @@ public:
   uint m_src_line;
 
 public:
+
   static void *operator new(size_t size, MEM_ROOT *mem_root) throw ()
   { return alloc_root(mem_root, size); }
   static void operator delete(void *ptr, MEM_ROOT *mem_root) {}
@@ -657,6 +670,38 @@ public:
 public:
 #ifdef WITH_WSREP
   void wsrep_report(bool debug);
+
+  const char* get_type_string()
+  {
+    switch(m_type)
+    {
+    case MDL_INTENTION_EXCLUSIVE:
+      return "MDL_INTENTION_EXCLUSIVE";
+    case MDL_SHARED:
+      return "MDL_SHARED";
+    case MDL_SHARED_HIGH_PRIO:
+      return "MDL_SHARED_HIGH_PRIO";
+    case MDL_SHARED_READ:
+      return "MDL_SHARED_READ";
+    case MDL_SHARED_WRITE:
+      return "MDL_SHARED_WRITE";
+    case MDL_SHARED_WRITE_LOW_PRIO:
+      return "MDL_SHARED_WRITE_LOW_PRIO";
+    case MDL_SHARED_UPGRADABLE:
+      return "MDL_SHARED_UPGRADABLE";
+    case MDL_SHARED_READ_ONLY:
+      return "MDL_SHARED_READ_ONLY";
+    case MDL_SHARED_NO_WRITE:
+      return "MDL_SHARED_NO_WRITE";
+    case MDL_SHARED_NO_READ_WRITE:
+      return "MDL_SHARED_NO_READ_WRITE";
+    case MDL_EXCLUSIVE:
+      return "MDL_EXCLUSIVE";
+    case MDL_TYPE_END:
+      return "NULL";
+    }
+    return "NULL";
+  }
 #endif /* WITH_WSREP */
   bool has_pending_conflicting_lock() const;
 
@@ -922,6 +967,8 @@ public:
 #endif /* WITH_WSREP */
   bool has_locks(MDL_key::enum_mdl_namespace mdl_namespace) const;
 
+  bool has_locks_waited_for() const;
+
   MDL_savepoint mdl_savepoint()
   {
     return MDL_savepoint(m_tickets[MDL_STATEMENT].front(),
