@@ -1,18 +1,27 @@
-/* Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; version 2 of the License.
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
+
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+
+#include "sql_class.h"
 #include <AclAPI.h>
 #include <accctrl.h>
 #include <errno.h>
@@ -48,24 +57,77 @@ bool is_existing_windows_group_name(const char *group_name)
   return true;
 }
 
-bool is_valid_named_pipe_full_access_group(const char *group_name)
+/*
+return false on successfully checking group, true on error.
+*/
+static bool check_windows_group_for_everyone(const char *group_name,
+                                             bool *is_everyone_group)
 {
-  // Treat the DEFAULT_NAMED_PIPE_FULL_ACCESS_GROUP value
-  // as a special case: we (later) convert it to the "world" SID
-  if (strcmp(group_name, DEFAULT_NAMED_PIPE_FULL_ACCESS_GROUP) == 0)
-  {
-    return true;
-  }
-
-  if (!group_name || group_name[0] == '\0' ||
-    is_existing_windows_group_name(group_name))
-  {
-    return true;
-  }
-  else
+  *is_everyone_group= false;
+  if (!group_name || group_name[0] == '\0')
   {
     return false;
   }
+
+  if (strcmp(group_name, DEFAULT_NAMED_PIPE_FULL_ACCESS_GROUP) == 0)
+  {
+    *is_everyone_group= true;
+    return false;
+  }
+  else
+  {
+    TCHAR last_error_msg[256];
+    // First, let's get a SID for the given group name...
+    BYTE soughtSID[SECURITY_MAX_SID_SIZE]= {0};
+    DWORD size_sought_sid= SECURITY_MAX_SID_SIZE;
+    BYTE worldSID[SECURITY_MAX_SID_SIZE]= {0};
+    DWORD size_world_sid= SECURITY_MAX_SID_SIZE;
+    char referencedDomainName[MAX_PATH];
+    DWORD size_referencedDomainName= MAX_PATH;
+    SID_NAME_USE sid_name_use;
+
+    if (!LookupAccountName(NULL, group_name, soughtSID, &size_sought_sid,
+                         referencedDomainName, &size_referencedDomainName,
+                         &sid_name_use))
+    {
+      return false;
+    }
+
+    if (!CreateWellKnownSid(WinWorldSid, NULL, worldSID, &size_world_sid))
+    {
+      DWORD last_error_num= GetLastError();
+      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
+                    NULL, last_error_num,
+                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), last_error_msg,
+                    sizeof(last_error_msg) / sizeof(TCHAR), NULL);
+      my_printf_error(ER_UNKNOWN_ERROR,
+                      "check_windows_group_for_everyone, CreateWellKnownSid failed: %s",
+                      MYF(0), last_error_msg);
+      return true;
+    }
+
+    *is_everyone_group= EqualSid(soughtSID, worldSID);
+    return false;
+  }
+}
+
+bool is_valid_named_pipe_full_access_group(const char *group_name)
+{
+  if (!group_name || group_name[0] == '\0'){
+    return true;
+  }
+
+  bool is_everyone_group= false;
+
+  if(check_windows_group_for_everyone(group_name, &is_everyone_group)){
+    return false;
+  }
+
+  if (is_everyone_group || is_existing_windows_group_name(group_name))
+  {
+    return true;
+  }
+  return false;
 }
 
 // return false on success, true on failure.
@@ -81,6 +143,23 @@ bool my_security_attr_add_rights_to_group(SECURITY_ATTRIBUTES *psa,
   DWORD size_referencedDomainName= MAX_PATH;
   SID_NAME_USE sid_name_use;
 
+  bool is_everyone_group= false;
+
+  if(check_windows_group_for_everyone(group_name, &is_everyone_group)){
+    return true;
+  }
+
+  if (is_everyone_group)
+  {
+    sql_print_warning(ER_DEFAULT(WARN_NAMED_PIPE_ACCESS_EVERYONE), group_name);
+    if (current_thd)
+    {
+      push_warning_printf(current_thd, Sql_condition::SL_WARNING,
+        WARN_NAMED_PIPE_ACCESS_EVERYONE,
+        ER(WARN_NAMED_PIPE_ACCESS_EVERYONE), group_name);
+    }
+  }
+
   // Treat the DEFAULT_NAMED_PIPE_FULL_ACCESS_GROUP value
   // as a special case: we  convert it to the "world" SID
   if (strcmp(group_name, DEFAULT_NAMED_PIPE_FULL_ACCESS_GROUP) == 0)
@@ -92,8 +171,9 @@ bool my_security_attr_add_rights_to_group(SECURITY_ATTRIBUTES *psa,
                     NULL, last_error_num,
                     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), last_error_msg,
                     sizeof(last_error_msg) / sizeof(TCHAR), NULL);
-      sql_print_error("my_security_attr_add_rights_to_group, CreateWellKnownSid failed: %s",
-                      last_error_msg);
+      my_printf_error(ER_UNKNOWN_ERROR,
+                      "my_security_attr_add_rights_to_group, CreateWellKnownSid failed: %s",
+                      MYF(0), last_error_msg);
       return true;
     }
   }
@@ -243,7 +323,9 @@ HANDLE create_server_named_pipe(SECURITY_ATTRIBUTES **ppsec_attr,
 
     if (last_error_num == ERROR_ACCESS_DENIED)
     {
-      sql_print_error("Can't start server : Named Pipe \"%s\" already in use.", name);
+      my_printf_error(ER_CANT_START_SERVER_NAMED_PIPE,
+        ER_DEFAULT(ER_CANT_START_SERVER_NAMED_PIPE),
+        MYF(ME_FATALERROR), name);
     }
     else
     {
