@@ -565,6 +565,43 @@ static void wsrep_pfs_instr_cb(
 }
 #endif /* HAVE_PSI_INTERFACE */
 
+void WSREP_LOG(void (*fun)(const char* fmt, ...), const char* fmt, ...)
+{
+  /* Allocate short buffer from stack. If the vsnprintf() return value
+     indicates that the message was truncated, a new buffer will be allocated
+     dynamically and the message will be reprinted. */
+  char msg[128] = {'\0'};
+  va_list arglist;
+  va_start(arglist, fmt);
+  int n= vsnprintf(msg, sizeof(msg) - 1, fmt, arglist);
+  va_end(arglist);
+  if (n < 0)
+  {
+    sql_print_warning("WSREP: Printing message failed");
+  }
+  else if (n < (int)sizeof(msg))
+  {
+    fun("WSREP: %s", msg);
+  }
+  else
+  {
+    try
+    {
+      std::vector<char> dynbuf(std::max(n, 4096));
+      va_start(arglist, fmt);
+      (void)vsnprintf(&dynbuf[0], dynbuf.size() - 1, fmt, arglist);
+      va_end(arglist);
+      dynbuf[dynbuf.size() - 1] = '\0';
+      fun("WSREP: %s", &dynbuf[0]);
+    }
+    catch (const std::bad_alloc&)
+    {
+      /* Memory allocation for vector failed, print truncated message. */
+      fun("WSREP: %s", msg);
+    }
+  }
+}
+
 static void wsrep_log_cb(wsrep_log_level_t level, const char *msg) {
   switch (level) {
   case WSREP_LOG_INFO:
