@@ -1,13 +1,20 @@
 /* Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  it under the terms of the GNU General Public License, version 2.0,
+  as published by the Free Software Foundation.
+
+  This program is also distributed with certain software (including
+  but not limited to OpenSSL) that is licensed under separate terms,
+  as designated in a particular file or component or in included license
+  documentation.  The authors of MySQL hereby grant you an additional
+  permission to link the program and your derivative works with the
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
@@ -525,6 +532,7 @@ PFS_thread* create_thread(PFS_thread_class *klass, const void *identity,
     pfs->m_stmt_lock.set_allocated();
     pfs->m_session_lock.set_allocated();
     pfs->set_enabled(true);
+    pfs->m_disable_instrumentation= false;
     pfs->set_history(true);
     pfs->m_class= klass;
     pfs->m_events_waits_current= & pfs->m_events_waits_stack[WAIT_STACK_BOTTOM];
@@ -1725,21 +1733,18 @@ void aggregate_thread_status(PFS_thread *thread,
 
   if (likely(safe_account != NULL))
   {
-    safe_account->aggregate_status_stats(&thd->status_var,
-                                         thd->status_var_aggregated);
+    safe_account->aggregate_status_stats(&thd->status_var);
     return;
   }
 
   if (safe_user != NULL)
   {
-    safe_user->aggregate_status_stats(&thd->status_var,
-                                      thd->status_var_aggregated);
+    safe_user->aggregate_status_stats(&thd->status_var);
   }
 
   if (safe_host != NULL)
   {
-    safe_host->aggregate_status_stats(&thd->status_var,
-                                      thd->status_var_aggregated);
+    safe_host->aggregate_status_stats(&thd->status_var);
   }
   return;
 }
